package com.sun.jndi.ldap.sasl;

import java.util.Vector;
import java.util.StringTokenizer;
import javax.naming.NamingException;
import java.io.IOException;
import javax.security.sasl.SaslClient;
import javax.security.sasl.SaslException;
import javax.naming.AuthenticationNotSupportedException;
import javax.naming.AuthenticationException;
import javax.security.sasl.Sasl;
import javax.security.auth.callback.CallbackHandler;
import com.sun.jndi.ldap.LdapResult;
import javax.naming.ldap.Control;
import java.util.Hashtable;
import com.sun.jndi.ldap.Connection;
import com.sun.jndi.ldap.LdapClient;

public final class LdapSasl
{
    private static final String SASL_CALLBACK = "java.naming.security.sasl.callback";
    private static final String SASL_AUTHZ_ID = "java.naming.security.sasl.authorizationId";
    private static final String SASL_REALM = "java.naming.security.sasl.realm";
    private static final int LDAP_SUCCESS = 0;
    private static final int LDAP_SASL_BIND_IN_PROGRESS = 14;
    private static final byte[] NO_BYTES;
    
    private LdapSasl() {
    }
    
    public static LdapResult saslBind(LdapClient ldapClient, Connection connection, String serverName, String s, Object o, String s2, Hashtable<?, ?> props, Control[] array) throws IOException, NamingException {
        boolean n = false;
        CallbackHandler cbh = (props != null) ? ((CallbackHandler)props.get("java.naming.security.sasl.callback")) : null;
        if (cbh == null) {
            cbh = new DefaultCallbackHandler(s, o, (String)props.get("java.naming.security.sasl.realm"));
            n = true;
        }
        String authorizationId = (props != null) ? ((String)props.get("java.naming.security.sasl.authorizationId")) : null;
        String[] saslMechanismNames = getSaslMechanismNames(s2);
        try {
            SaslClient saslClient = Sasl.createSaslClient(saslMechanismNames, authorizationId, "ldap", serverName, props, cbh);
            if (saslClient == null) {
                throw new AuthenticationNotSupportedException(s2);
            }
            String mechanismName = saslClient.getMechanismName();
            LdapResult ldapResult = ldapClient.ldapBind(null, saslClient.hasInitialResponse() ? saslClient.evaluateChallenge(LdapSasl.NO_BYTES) : null, array, mechanismName, true);
            while (!saslClient.isComplete() && (ldapResult.status == 14 || ldapResult.status == 0)) {
                byte[] evaluateChallenge = saslClient.evaluateChallenge((ldapResult.serverCreds != null) ? ldapResult.serverCreds : LdapSasl.NO_BYTES);
                if (ldapResult.status == 0) {
                    if (evaluateChallenge != null) {
                        throw new AuthenticationException("SASL client generated response after success");
                    }
                    break;
                }
                else {
                    ldapResult = ldapClient.ldapBind(null, evaluateChallenge, array, mechanismName, true);
                }
            }
            if (ldapResult.status == 0) {
                if (!saslClient.isComplete()) {
                    throw new AuthenticationException("SASL authentication not complete despite server claims");
                }
                String s3 = (String)saslClient.getNegotiatedProperty("javax.security.sasl.qop");
                if (s3 != null && (s3.equalsIgnoreCase("auth-int") || s3.equalsIgnoreCase("auth-conf"))) {
                    connection.replaceStreams(new SaslInputStream(saslClient, connection.inStream), new SaslOutputStream(saslClient, connection.outStream));
                }
                else {
                    saslClient.dispose();
                }
            }
        }
        catch (SaslException rootCause) {
            AuthenticationException o2 = new AuthenticationException(s2);
            o2.setRootCause(rootCause);
            throw o2;
        }
        finally {
            if (!(n == 0)) {
                ((DefaultCallbackHandler)cbh).clearPassword();
            }
        }
    }
    
    private static String[] getSaslMechanismNames(String str) {
        StringTokenizer stringTokenizer = new StringTokenizer(str);
        Vector vector = new Vector(10);
        while (stringTokenizer.hasMoreTokens()) {
            vector.addElement(stringTokenizer.nextToken());
        }
        String[] array = new String[vector.size()];
        for (int i = 0; i < vector.size(); ++i) {
            array[i] = (String)vector.elementAt(i);
        }
        return array;
    }
    
    static {
        NO_BYTES = new byte[0];
    }
}
